/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "Page.h"
#include "Namer.h"
#include "Macros.h"
#include "Presentation.h"
#include <iostream>

using namespace std;
using namespace slideshow;


Page::Page (Page *base_)
{
	Number = 0;
	
	base = base_;
	owned = NULL;
	last_writer = NULL;
	drawing     = NULL;
	notes       = "";
	
	stage_adding  = 0;
	stage_current = 0;
	stage_max     = 0;
	
	SetFisrtStage();
}


Page::~Page()
{
	if (drawing) {
		delete drawing;
	}
}


void
Page::Draw (SDL_Surface *screen)
{
	SS_ASSERT(screen);

	if (base) {
		base->Draw(screen);
	}
	
	for (DrawableList::const_iterator it=drawables.begin(); it!=drawables.end(); ++it) {
		if ((*it)->stage <= stage_current) {
			(*it)->Draw (screen);
		}
	}
}


void
Page::Add (Drawable *item)
{
	item->stage = stage_adding;
	drawables.push_back (item);
}


void
Page::Save (SDL_Surface *screen, const char *filename)
{
	   assert (filename);

	   // Say cheeseeeeeeeeeeeee :-)
	   for (DrawableList::const_iterator it=drawables.begin(); it!=drawables.end(); ++it) {
		   (*it)->SayCheese();
	   }
	   
	   Draw (screen);

	   if (SDL_SaveBMP (screen, filename) < 0) {
		   cerr << "Can't write image " << filename << endl;
	   }	   
}


void
Page::SetBase (Page *base_)
{
	base = base_;
}


void
Page::SetPresentaion (Presentation *presentation)
{
	owned = presentation;
}


void
Page::AddNote (const char *note)
{
	notes += note;
}


Image *
Page::AddNewImage (char *filename, int x, int y)
{
	   Image *ni = new Image(filename);
	   if (x != -1) ni->x = x;
	   if (y != -1) ni->y = y;

	   Add (ni);

	   return ni;
}


Image *
Page::AddNewImageScrolled (char *filename, int dir, int another, int stop_at)
{
	   Image *ni;

	   if (dir == NORTH_SOUTH || dir == SOUTH_NORTH) {
			 ni = AddNewImage(filename, another, -1);
	   } else {
			 ni = AddNewImage(filename, -1, another);
	   }

	   ni->SetScroll (dir, stop_at);

	   return ni;
}


Text *
Page::AddNewText (char *fontname, char *text, int x, int y, int color)
{
	if (owned == NULL) {
		cerr << "Slide wasn't added to a presentation" << endl;
	}
	
	TTF_Font *font = owned->Fonts[fontname];
	Text *nt = new Text(font);

	if (text)    nt->SetText(text);
	if (x != -1) nt->x = x;
	if (y != -1) nt->y = y;
	
	nt->SetColor (color);
	
	Add (nt);
	
	return nt;
}


TextDouble *
Page::AddNewTextDouble (char *fontname, char *text, int x, int y, int c1, int c2)
{
	if (owned == NULL) {
		cerr << "Slide wasn't added to a presentation" << endl;
	}
	
	TTF_Font *font = owned->Fonts[fontname];
	
	TextDouble *nt = new TextDouble(font, c1, c2);

	if (text)    nt->SetText(text);
	if (x != -1) nt->x = x;
	if (y != -1) nt->y = y;
	
	Add (nt);
	
	return nt;
}


Text *
Page::AddNewTextScrolled (char *fontname, char *text, int dir, int another, int stop_at, int color)
{
	   Text *t;

	   if (dir == NORTH_SOUTH || dir == SOUTH_NORTH) {
			 t = AddNewText(fontname, text, another, -1, color);
	   } else {
			 t = AddNewText(fontname, text, -1, another, color);
	   }

	   t->SetScroll (dir, stop_at);

	   return t;
}


TextDouble *
Page::AddNewTextDoubleScrolled (char *fontname, char *text, int dir, int another, int stop_at, int c1, int c2)
{
	TextDouble *t;
	
	if (dir == NORTH_SOUTH || dir == SOUTH_NORTH) {
		t = AddNewTextDouble(fontname, text, another, -1, c1, c2);
	} else {
		t = AddNewTextDouble(fontname, text, -1, another, c1, c2);
	}

	t->SetScroll (dir, stop_at);
	
	return t;
}


Mpeg *
Page::AddNewMpeg (char *filename, int x, int y)
{
	Mpeg *nm;

	nm = new Mpeg(filename);
	if (x != -1) nm->x = x;
	if (y != -1) nm->y = y;
		
	Add(nm);

	return nm;
}


Mpeg *
Page::AddNewMpegScrolled (char *filename, int dir, int another, int stop_at)
{
	Mpeg *nm;

	if (dir == NORTH_SOUTH || dir == SOUTH_NORTH) {
		nm = AddNewMpeg(filename, another, -1);
	} else {
		nm = AddNewMpeg(filename, -1, another);
	}

	nm->SetScroll (dir, stop_at);

	return nm;
}


Gradient *
Page::AddNewGradient (unsigned int color1, unsigned int color2,SlideShowGradient type)
{
	Gradient *ng;

	ng = new Gradient (color1, color2,type);
	Add(ng);

	return ng;
}


Stars *
Page::AddNewStars (void) 
{
	Stars *ns;

	if(!owned) {
		cerr << "Stars: Page not on a presentation! Using default screen size...\n";
		ns = new Stars();
	} else {
		ns = new Stars(owned->GetWidth(), owned->GetHeight());
	}

	Add(ns);

	return ns;
}


Plasma *
Page::AddNewPlasma (void) 
{
	Plasma *np;

	if(!owned) {
		cerr << "Plasma: Page not on a presentation! Using default screen size...\n";
		np = new Plasma();
	} else {
		np = new Plasma(owned->GetWidth(), owned->GetHeight());
	}

	Add(np);

	return np;
}


Fires *
Page::AddNewFires(void) 
{
	Fires *nf;

	if(!owned) {
		cerr << "Fires: Page not on a presentation! Using default screen size...\n";
		nf = new Fires();
	} else {
		nf = new Fires(owned->GetWidth(), owned->GetHeight());
	}

	Add(nf);

	return nf;
}


Eraser *
Page::AddNewEraser (unsigned int color)
{
	Eraser *ne;

	ne = new Eraser (color);
	Add (ne);

	return ne;
}


EffectDarkness *
Page::AddNewEffectDarkness (Drawable *drawable)
{
	EffectDarkness *ned;

	ned = new EffectDarkness (drawable);
	Add(ned);

	return ned;
}



Spinner *
Page::AddNewSpinner(Drawable *drawable, int degrees_second)
{
    Spinner *spn;
    
    spn = new Spinner(drawable);
    spn->DegreesPerSecond = degrees_second;
    Add(spn);
    
    return spn;   
}


Writer *
Page::AddNewWriter (char *font, int x, int y, int color, int color2)
{
	if (owned == NULL) {
		cerr << "Slide wasn't added to a presentation" << endl;
	}

	if (font == NULL) {
		font = owned->Last_writer_fontname;
	} else {
		owned->Last_writer_fontname = font;
	}

	if (x == SS_UNDEFINED_COLOR) {
		x = owned->Last_writer_posx;
	} else {
		owned->Last_writer_posx = x;
	}

	if (y == SS_UNDEFINED_COLOR) {
		y = owned->Last_writer_posy;
	} else {
		owned->Last_writer_posy = y;
	}
	
	if (color == SS_UNDEFINED_COLOR) {
		color = owned->Last_writer_color1;
	} else {
		owned->Last_writer_color1 = color;
	}

	if (color2 == SS_UNDEFINED_COLOR) {
		color2 = owned->Last_writer_color2;
	} else {
		owned->Last_writer_color2 = color2;
	}

	FontVault *fontvault = &(owned->Fonts);
	
	Writer *nw = new Writer (fontvault, color, color2);
	nw->SetDefaultFont(font);
	nw->x = x;
	nw->y = y;

	Add (nw);
	last_writer = nw;
	   
	return nw;
}


Page*
Page::operator+= (char *text)
{
	   // "return this" for python binding

	   if (last_writer == NULL) {
		   last_writer = AddNewWriter();
	   }

	   last_writer->AppendLine (text);

	   return this;
}



bool
Page::NeedRedraw (void)
{
/* UGLY:
 * If some item of the screen needs be repainted,
 * it repaints all the screen.
 */
	if (base) {
		if (base->NeedRedraw())
			return true;
	}
	
	for (DrawableList::const_iterator it=drawables.begin(); it!=drawables.end(); ++it) {
		if ((*it)->stage <= stage_current) {
			if ((*it)->NeedRedraw()) 
				return true;
		}
	}
	
	return false;
}



void
Page::AddStop (float secs)
{
	times.push_back(secs);
	stage_adding++;
}


bool
Page::LastStage (void)
{
	return (stage_current == stage_adding);
}


bool
Page::FirstStage (void)
{
	return (stage_current == 0);
}


void
Page::SetPrevStage (void)
{
	--stage_current;
	time(&stage_current_begin);
}


void
Page::SetNextStage (void)
{
	++stage_current;
	time(&stage_current_begin);
}


void
Page::SetFisrtStage (void)
{
	stage_current = 0;
	time(&stage_current_begin);
}


bool
Page::StageFinished(void)
{
	/* A stage-less page never ends
	 */
	if (stage_adding == 0) {
		return false;
	}


	time_t now;	
	time(&now);
	
	double elapse = difftime(now, stage_current_begin);

	return (elapse >= times[stage_current]);
}


void
Page::NotifyLeaving (void)
{
	for (DrawableList::const_iterator it=drawables.begin(); it!=drawables.end(); ++it) {
		(*it)->On_LeavingPage();
	}	
}
	

void
Page::NotifyJoining (void)
{
	for (DrawableList::const_iterator it=drawables.begin(); it!=drawables.end(); ++it) {
		(*it)->On_JoiningPage();
	}
}


void
Page::DrawingAddPoint (Uint16 x, Uint16 y)
{
	if (drawing == NULL) {
		drawing = new HandDrawing();
		Add(drawing);
		drawing->stage = 0;
	}

	drawing->AddPoint(x,y);
}


string
Page::ExportPython (void) const
{
	Namer namer;

	string tmp;

	for (DrawableList::const_iterator it=drawables.begin(); it!=drawables.end(); ++it) {

		cout << "*it" << Number <<  endl;
		string *name = namer.GetName(*it, Number);
		cout << name << endl;

		tmp += (*it)->ToPython();
	}

	return tmp;
}


string
Page::ExportXML (void) const
{
	string tmp;
	for (DrawableList::const_iterator it=drawables.begin(); it!=drawables.end(); ++it) {
		tmp += (*it)->ToXML();
	}
	return tmp;
}


string
Page::ExportCPP (void) const
{
	string tmp;
	for (DrawableList::const_iterator it=drawables.begin(); it!=drawables.end(); ++it) {
		(*it)->ToCPP();
	}
	return tmp;
}


