/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* SlideShow
 *
 * Authors:
 *      Alvaro Lopez Ortega <alvaro@alobbs.com>
 *
 * Copyright (C) 2002-2003 Alvaro Lopez Ortega
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <time.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <iostream>
#include <SDL_ttf.h>

#include "Macros.h"
#include "Utilities.h"
#include "Presentation.h"
#include "Loader.h"
#include "XmlLoader.h"
#include "MgpLoader.h"
#include "HtmlExporter.h"


using namespace std;


#define PRESENTATION_DEFAULT_SDL_FLAGS SDL_DOUBLEBUF|SDL_ANYFORMAT
#define WRITER_COLOR_UNSET 0xDEADBEEF


Presentation::Presentation (int extra_sdl_flags, int video_mode, int xsz, int ysz)
{
	   page_actual = NULL;
	   force_redraw_page = false;	   
	   last_page_number = 0;
	   
	   Last_writer_color1   = WRITER_COLOR_UNSET;
	   Last_writer_color2   = WRITER_COLOR_UNSET;
	   Last_writer_posx     = WRITER_COLOR_UNSET;
	   Last_writer_posy     = WRITER_COLOR_UNSET;
	   Last_writer_fontname = NULL;

	   
	   if (video_mode == VIDEO_NOINIT) {
		   return;
	   }

	   
	   // SDL Init
	   if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER) < 0 ) {
		   cerr << "Unable to init SDL: " << SDL_GetError() << endl;
		   exit(1);
	   }

	   // TTF Font engine init
	   if (TTF_Init() < 0) {
		   cerr << "Unable to init TTF: " << SDL_GetError() << endl;
		   exit(2);
	   }

	   
	   // Video init	   
	   if (video_mode == VIDEO_ASCII) {
		   setenv("SDL_VIDEODRIVER", "aalib", 1);
	   }
	   	   
	   atexit(SDL_Quit);

	   // Flags
	   int flags = PRESENTATION_DEFAULT_SDL_FLAGS;
	   if (extra_sdl_flags) {
		   flags |= extra_sdl_flags;
	   }

	   // Screen
	   screen = SDL_SetVideoMode (DEFAULT_WIDTH, DEFAULT_HEIGHT, 0, flags);
	   if (screen == NULL) {
		   cerr << "Unable to set " << DEFAULT_WIDTH << " " << DEFAULT_HEIGHT 
			<< "video mode: " << SDL_GetError() << endl;
		   exit(1);
	   }

	   SDL_WM_SetCaption(PACKAGE, NULL);

	   width = xsz;
	   height = ysz;
}


Presentation::~Presentation()
{
	   if (screen) {
			 SDL_FreeSurface(screen);
	   }
}


void
Presentation::__SetScreenSurface (SDL_Surface *s)
{
	SS_ASSERT (s);
	screen = s;
}

bool
Presentation::DoMainloopIteration (void)
{
	bool      exit    = false;
	bool      drawing = false;
	SDL_Event event;

	while (SDL_PollEvent(&event) && (!exit)) {
			switch (event.type) {
			case SDL_KEYUP:
				switch (event.key.keysym.sym) {
				case SDLK_q:
					exit = true;
					break;
					
				case SDLK_x:
					if (mouse_is_active) {
						MouseSetInactive();
					} else {
						MouseSetActive();
					}
					break;
					
				case SDLK_f:
					SDL_WM_ToggleFullScreen (screen);
					break;
					
				case SDLK_s:
					WriteBMPs();
					break;
					
				case SDLK_d: 
					WriteCurrentBMP();
					break;
					
				case SDLK_RIGHT:
				case SDLK_SPACE:
					SetNextStage();
					force_redraw_page = true;
					break;
					
				case SDLK_LEFT:
				case SDLK_BACKSPACE:
					SetPrevStage();
					force_redraw_page = true;
					break;
					
				case SDLK_PAGEDOWN:
					SetNextPage();
					force_redraw_page = true;
					break;
					
				case SDLK_PAGEUP:
					SetPrevPage();
					force_redraw_page = true;
					break;
					
				default:
					break;
				}
				break;

			case SDL_MOUSEMOTION:
				if (event.motion.state == 1) {
					if (!drawing) {
						page_actual->DrawingAddPoint (0xFFFF, 0xFFFF);
					}
					page_actual->DrawingAddPoint (event.motion.x, event.motion.y);
				}
				drawing = (event.motion.state == 1);
				break;
				
			case SDL_QUIT:
				exit = true;
				break;
				
			default:
				break;
			}
		}

	if (page_actual == NULL) {
		cerr  << "ERROR: Empty presentation" << endl;
		return true;
	}
	
	
	/* Check the elapse time */
	if (page_actual->StageFinished()) {
		if (!page_actual->LastStage()) {
			page_actual->SetNextStage();
			force_redraw_page = true;
		}
	}
	
	if (page_actual->NeedRedraw() || force_redraw_page) {
		force_redraw_page = false;
		page_actual->Draw(screen);
		SDL_Flip(screen);
	} else {
		usleep(10);
	}
	
	return exit;
}


void
Presentation::Run (void)
{	
	if (!page_actual) {
		cerr << "Presentation empty" << endl;
		return;
	}
	
	while (!DoMainloopIteration());
}


void
Presentation::Add (Page *page)
{
	   pages.push_back(page);

	   if (!page_actual) {
			 page_actual = page;
	   }

	   page->SetPresentaion(this);
	   page->Number = ++last_page_number;
}


void
Presentation::SetNextStage (void)
{
	if (page_actual->LastStage()) {
		std::list<Page*>::const_iterator last = pages.end();
		--last;
		
		if (page_actual != (*last)) {
			SetNextPage();
			page_actual->SetFisrtStage();
		}

	} else {
		page_actual->SetNextStage();
	}
}


void
Presentation::SetNextPage (void)
{
	page_actual->NotifyLeaving();

	for (std::list<Page*>::const_iterator it=pages.begin(), last=pages.end(), next; it!=last; ++it) {
		++it; next=it; --it; // puag..
		if ((page_actual == (*it)) && (next != last)) {
			page_actual = *(next);			
			force_redraw_page = true;
			page_actual->NotifyJoining();
			return;
		}
	}
}


void
Presentation::SetPrevStage (void)
{
	if (page_actual->FirstStage()) {
		std::list<Page*>::const_iterator begin = pages.begin();

		if (page_actual != (*begin)) {
			SetPrevPage();
		}
		
	} else {
		page_actual->SetPrevStage();
	}
}


void
Presentation::SetPrevPage (void)
{
	page_actual->NotifyLeaving();

	for (std::list<Page*>::const_iterator it=pages.begin(), last=pages.end(); it!=last; ++it) {
		if ((page_actual == (*it))) { 
			if (page_actual == *pages.begin()) return;
			page_actual = *(--it);
			force_redraw_page = true;
			page_actual->NotifyJoining();
			return;
		}
	}
}


int
Presentation::WriteBMPs (const char *prefix)
{
	   const char *p = (prefix) ? prefix : "presentation-";
	   char filename[200];
	   int n = 1;

	   for (std::list<Page*>::const_iterator it=pages.begin(); it!=pages.end(); ++it) {
			 snprintf (filename, 200, "%s%03i.bmp", p, n);
			 (*it)->Save (screen, filename);
			 SDL_Flip(screen);
			 n++;
	   }

	   return n;
}


void
Presentation::WriteCurrentBMP (const char *filename)
{
	   if (!page_actual) {
			 cerr << "No actual page. Can't dump BMP file" << endl;
			 return;
	   }

	   
	   time_t      rawtime;
	   struct tm * timeinfo;

	   time (&rawtime);
	   timeinfo = localtime (&rawtime);

	   if (!filename) {
			 char buf[50];	   
			 snprintf (buf, 50, "%s.bmp", asctime(timeinfo));
			 page_actual->Save (screen, buf);
	   } else {
			 page_actual->Save (screen, filename);
	   }
}


Page *
Presentation::AddNewPage (Page *base)
{
	   Page *np = new Page(base);
	   Add(np);
	   return np;
}


void
Presentation::MouseSetActive (void)
{
	mouse_is_active = true;
	
	SDL_ShowCursor(SDL_ENABLE);
	SDL_EventState(SDL_MOUSEMOTION, SDL_ENABLE);
	SDL_EventState(SDL_MOUSEBUTTONUP, SDL_ENABLE);
	SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_ENABLE);
}


void
Presentation::MouseSetInactive (void)
{	   
	mouse_is_active = false;

	SDL_ShowCursor(SDL_DISABLE);
	SDL_EventState(SDL_MOUSEMOTION, SDL_IGNORE);
	SDL_EventState(SDL_MOUSEBUTTONUP, SDL_IGNORE);
	SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_IGNORE);
}


bool
Presentation::IsEmpty (void)
{
	return (page_actual == NULL);
}


void
Presentation::Load (const char *filename)
{
	SS_ASSERT (filename);

	Loader *loader;
	int fn_len = strlen(filename);
	
	if (fn_len <= 4) {
		return;
	}

	if (endsWith (filename, ".xml")) {
		loader = new XmlLoader();
	} else if (endsWith (filename, ".mgp")) {
		loader = new MgpLoader();
	} else {
		cerr << "Unknown file type in file: " << filename 
		     << " (" << (char *)&filename[fn_len-4] << ")" << endl;
		return;
	}

	loader->Load(filename, this);
	delete loader;
}


void
Presentation::ExportHTML (const char *directory)
{
	HtmlExporter exporter;
	exporter.Write(this, directory);
}


void
Presentation::ExportPython (const char *filename)
{
	ofstream python(filename);

	python << "# Generated by " << PACKAGE << " " << VERSION << endl << endl;
	python << "from slideshow import *" << endl << endl;
	python << "p = Presentation()" << endl;
	
	for (std::list<Page*>::const_iterator it=pages.begin(), last=pages.end(), next; it!=last; ++it) {		
		python << (*it)->ExportPython() << endl << endl;
		 
	}

	python << "p.Run()" << endl;
	
	python.close();
}


void
Presentation::ExportXML (const char *filename)
{
	ofstream xml(filename);

	xml << "<!-- Abraham, please fill in it :-) -->" << endl;
	
	for (std::list<Page*>::const_iterator it=pages.begin(), last=pages.end(), next; it!=last; ++it) {		
		xml << (*it)->ExportXML() << endl << endl;
	}

	xml.close();
}


void
Presentation::ExportCPP (const char *filename)
{
	ofstream cpp(filename);
	
	cpp << "#include \"Presentation\"" << endl;
	cpp << "// Generated by " << PACKAGE << " " << VERSION << endl << endl;
	
	for (std::list<Page*>::const_iterator it=pages.begin(), last=pages.end(), next; it!=last; ++it) {		
		cpp << (*it)->ExportCPP() << endl << endl;			 
	}

	cpp.close();
}


bool
Presentation::RequireVersion  (unsigned short int version, 
			       unsigned short int subversion, 
			       unsigned short int microversion)
{
	bool ok = false;

	if (version < VERSION1) {
		ok = true;
	} else if (version == VERSION1) {
		if (subversion < VERSION2) {
			ok = true;
		} else if (subversion == VERSION2) {
			if (microversion <= VERSION3)
				ok = true;
		}
	}

	if (!ok) {
		cerr << "ERROR: *OLD* SlideShow version " << VERSION1 << "." << VERSION2 << "." << VERSION3
		     << " but version " << version << "." << subversion << "." << microversion << " is required" << endl;
	}

	return ok;
}


void
Presentation::ExecuteCommand (const char *command)
{
	FILE *file;

	file = popen (command, "w");
	if (file == NULL) {
		cerr << "ERROR: Cannot execute: \"" << command << "\"" << endl;
		return;
	}
	
	// FIXME:
	// fread() ?? 
	
	pclose (file);
}
